package com.wsdf.androidlockcontrol.activity;

import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.ListView;
import android.widget.Spinner;
import android.widget.TextView;

import com.blankj.utilcode.util.ConvertUtils;
import com.blankj.utilcode.util.ToastUtils;
import com.wsdf.androidlockcontrol.R;
import com.wsdf.androidlockcontrol.adapter.LockAdapter;
import com.wsdf.androidlockcontrol.base.BaseActivity;
import com.wsdf.androidlockcontrol.common.CommandManage;
import com.wsdf.androidlockcontrol.common.SerialUtils;
import com.wsdf.androidlockcontrol.entity.LockItem;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import android_serialport_api.SerialPort;
import butterknife.BindView;

public class MainActivity extends BaseActivity {

    private static final String TAG = "TAG";
    @BindView(R.id.tv_show)
    TextView tvShow;
    @BindView(R.id.sp_lock)
    Spinner spLock;
    @BindView(R.id.btn_main_open)
    Button btnMainOpen;
    @BindView(R.id.lv_lock)
    ListView lvLock;
    private LockAdapter la;
    private List<LockItem> lockItems;
    private Button btnOpen;
    private CheckBox cbAll;
    private int spSelectItem;
    private StringBuilder sb;
    private SimpleDateFormat sdfRecord;

    @Override
    protected int getLayoutId() {
        return R.layout.activity_main;
    }

    @Override
    protected void initView() {
        View view = getLayoutInflater().inflate(R.layout.lv_lock_footer, null);
        btnOpen = view.findViewById(R.id.btn_open);
        cbAll = view.findViewById(R.id.cb_lock_footer);
        lvLock.addFooterView(view);
    }

    @Override
    protected void initData() {
        sb = new StringBuilder();
        sdfRecord = new SimpleDateFormat("[HH:mm:ss] - ", Locale.CHINA);
        String[] item = getResources().getStringArray(R.array.lock_item);
        SerialPort serialPort = SerialUtils.getSerialPort();
        if (serialPort == null) {
            Log.e(TAG, "initData: 串口打开失败");
            return;
        }

        new Thread(() -> SerialUtils.setOnDataReceiveListener((bytes, size) -> runOnUiThread(() -> setDataShow(ConvertUtils.bytes2HexString(bytes), false)))).start();

        lockItems = new ArrayList<>();
        for (String s : item) {
            LockItem li = new LockItem();
            li.setName(s);
            li.setCheck(false);
            lockItems.add(li);
        }

        la = new LockAdapter(lockItems, this);
        lvLock.setAdapter(la);
    }

    @Override
    protected void addListener() {
        btnMainOpen.setOnClickListener(v -> {
            byte[] bytes = CommandManage.getIndexBytes(spSelectItem);
            new Thread(() -> SerialUtils.sendData(bytes)).start();
            setDataShow(ConvertUtils.bytes2HexString(bytes), true);
        });

        spLock.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                spSelectItem = position;
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        lvLock.setOnItemClickListener((parent, view, position, id) -> {
            LockItem li = lockItems.get(position);
            li.setCheck(!li.isCheck());
            la.notifyDataSetChanged();
            int flag = 0;
            for (LockItem item : lockItems) {
                if (item.isCheck()) {
                    flag++;
                }
            }
            cbAll.setChecked(flag == 12);
        });

        btnOpen.setOnClickListener(v -> {
            int flag = 0;
            for (LockItem item : lockItems) {
                if (item.isCheck()) {
                    flag++;
                }
            }
            if (flag == 0) {
                ToastUtils.showShort("请选择需要开启的锁！！！");
                return;
            }
            openLock();
        });

        cbAll.setOnClickListener(v -> {
            CheckBox cb = (CheckBox) v;
            for (LockItem li : lockItems) {
                li.setCheck(cb.isChecked());
            }
            la.notifyDataSetChanged();
        });
    }

    @Override
    protected void onDestroy() {
        SerialUtils.closeSerialPort();
        super.onDestroy();
    }

    private void setDataShow(String data, boolean isSend) {
        sb.append(sdfRecord.format(new Date()))
                .append(isSend ? "TX - " : "RX - ")
                .append(data)
                .append("\n");
        tvShow.setText(sb.toString());
        if (sb.length() > 4000) {
            sb.delete(0, 4000);
        }
    }

    private void openLock() {
        String str = "9001";
        StringBuilder sb1 = new StringBuilder();
        StringBuilder sb2 = new StringBuilder();
        for (int i = 0; i < 12; i++) {
            LockItem li = lockItems.get(i);
            if (i < 8) {
                sb1.append(li.isCheck() ? "1" : "0");
            } else {
                sb2.append(li.isCheck() ? "1" : "0");
            }
        }
        str += parseHexStr(sb1.toString()) + parseHexStr(sb2.toString());
        String strCheck = getXORCheck(str);
        str += strCheck.length() == 1 ? "0" + strCheck : strCheck;
        String finalStr = str;
        new Thread(() -> SerialUtils.sendData(ConvertUtils.hexString2Bytes(finalStr))).start();
        setDataShow(str, true);
    }

    //二进制转16进制
    private String parseHexStr(String binary) {
        String s = Integer.toHexString(Integer.parseInt(binary, 2)).toUpperCase();
        if (s.length() == 1) {
            s = "0" + s;
        }
        return s;
    }

    //异或校验
    private String getXORCheck(String hex) {
        String regex = "(.{2})";
        String replace = hex.replaceAll(regex, "$1 ");
        String[] st = replace.split(" ");
        int[] c = new int[st.length + 1];
        for (int i = 0; i < st.length; i++) {
            int i1 = Integer.parseInt(st[i], 16);
            c[i] = i1;
        }
        char x = 0;
        for (int value : c) {
            x ^= value;
        }
        return String.format("%x", (int) x).toUpperCase();
    }
}