package com.wsdf.androidlockcontrol.common;

import android.util.Log;

import com.blankj.utilcode.util.ConvertUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import android_serialport_api.SerialPort;

public class SerialUtils {

    private static final String TAG = "TAG";
    private static SerialPort serialPort;
    private static InputStream inputStream;
    private static OutputStream outputStream;
    private static ReadThread readThread;
    private static boolean Status;
    private static OnDataReceiveListener onDataReceiveListener;

    //开启串口
    public static SerialPort getSerialPort() {
        try {
            if (serialPort == null) {
                serialPort = new SerialPort(new File(Constants.PORT_S3), Constants.BAUD_RATE, 0);
                Log.e(TAG, "getSerialPort: 串口打开成功");
                inputStream = serialPort.getInputStream();
                outputStream = serialPort.getOutputStream();
                Status = true;
                if (readThread == null) {
                    readThread = new ReadThread();
                }
                if (!readThread.isAlive()) {
                    readThread.start();
                }
            }
        } catch (IOException e) {
            Log.e(TAG, "getSerialPort: 串口打开异常");
            e.printStackTrace();
        }
        return serialPort;
    }

    private static class ReadThread extends Thread {
        @Override
        public void run() {
            super.run();
            while (Status) {
                byte[] bytes = new byte[5];
                int size;
                try {
                    size = inputStream.read(bytes);
                    if (size > 0) {
                        Log.e(TAG, "run: 接收到的数据大小：" + size + "，接收到的数据：" + ConvertUtils.bytes2HexString(bytes));
                        if (onDataReceiveListener != null) {
                            onDataReceiveListener.onDataReceive(bytes, size);
                        }
                    }
                } catch (IOException e) {
                    Log.e(TAG, "run: 串口数据读取异常" + e.toString());
                    e.printStackTrace();
                }
            }
        }
    }

    public static void sendData(byte[] values) {
        if (values.length > 0) {
            if (outputStream != null) {
                try {
                    outputStream.write(values);
                    outputStream.flush();
                    Log.e(TAG, "sendData: 数据发送成功->" + ConvertUtils.bytes2HexString(values));
                } catch (IOException e) {
                    Log.e(TAG, "sendData: 数据发送异常->" + e.toString());
                }
            }
        }
    }

    //接口
    public interface OnDataReceiveListener {
        void onDataReceive(byte[] bytes, int size);
    }

    public static void setOnDataReceiveListener(OnDataReceiveListener dataReceiveListener) {
        onDataReceiveListener = dataReceiveListener;
    }

    //关闭串口
    public static void closeSerialPort() {
        try {
            if (serialPort != null) {
                Status = false;
                inputStream.close();
                outputStream.close();
                serialPort.close();
                serialPort = null;
            }
            Log.e(TAG, "closeSerialPort: 关闭串口成功");
        } catch (IOException e) {
            Log.e(TAG, "closeSerialPort: 关闭串口异常" + e.toString());
            e.printStackTrace();
        }
    }

}
